==> python-ast-0.0.4.tgz: package/PythonFile.js <==
import { AstNode } from "./core/AstNode";
export class PythonFile extends AstNode {
    moduleName;
    path;
    name;
    statements = [];
    constructor({ moduleName, path, name }) {
        super();
        this.moduleName = moduleName;
        this.path = path;
        this.name = name;
    }
    addStatement(statement) {
        this.statements.push(statement);
        this.inheritReferences(statement);
    }
    write(writer) {
        this.writeImports(writer);
        this.statements.forEach((statement, idx) => {
            statement.write(writer);
            writer.newLine();
            if (idx < this.statements.length - 1) {
                writer.newLine();
            }
        });
    }
    /*******************************
     * Helper Methods
     *******************************/
    getImportName(reference) {
        const name = reference.name;
        const alias = reference.alias;
        return `${name}${alias ? ` as ${alias}` : ""}`;
    }
    writeImports(writer) {
        // Deduplicate references by their fully qualified paths
        const uniqueReferences = new Map();
        for (const reference of this.references) {
            const fullyQualifiedPath = reference.getFullyQualifiedModulePath();
            const existingRefs = uniqueReferences.get(fullyQualifiedPath);
            const referenceName = reference.name;
            if (existingRefs) {
                if (!existingRefs.referenceNames.has(referenceName)) {
                    existingRefs.references.push(reference);
                    existingRefs.referenceNames.add(referenceName);
                }
            }
            else {
                uniqueReferences.set(fullyQualifiedPath, {
                    modulePath: reference.modulePath,
                    references: [reference],
                    referenceNames: new Set([referenceName])
                });
            }
        }
        for (const [fullyQualifiedPath, { modulePath, references }] of uniqueReferences) {
            const refModulePath = modulePath;
            if (refModulePath[0] === this.moduleName) {
                // Relativize the import
                // Calculate the common prefix length
                let commonPrefixLength = 0;
                while (commonPrefixLength < this.path.length &&
                    commonPrefixLength < refModulePath.length &&
                    this.path[commonPrefixLength] === refModulePath[commonPrefixLength]) {
                    commonPrefixLength++;
                }
                // Calculate the number of levels to go up
                const levelsUp = this.path.length - commonPrefixLength;
                // Build the relative import path
                let relativePath = levelsUp > 0 ? ".".repeat(levelsUp) : ".";
                relativePath += refModulePath.slice(commonPrefixLength).join(".");
                // Write the relative import statement
                writer.write(`from ${relativePath} import ${references.map((ref) => this.getImportName(ref)).join(", ")}`);
            }
            else {
                // Use fully qualified path
                writer.write(`from ${fullyQualifiedPath} import ${references.map((ref) => this.getImportName(ref)).join(", ")}`);
            }
            writer.newLine();
        }
        if (Object.keys(uniqueReferences).length > 0) {
            writer.newLine();
        }
    }
}

==> python-ast-0.0.10.tgz: package/PythonFile.js <==
import { AstNode } from "./core/AstNode";
export class PythonFile extends AstNode {
    path;
    isInitFile;
    statements = [];
    constructor({ path, isInitFile = false }) {
        super();
        this.path = path;
        this.isInitFile = isInitFile;
    }
    addStatement(statement) {
        this.statements.push(statement);
        this.inheritReferences(statement);
    }
    write(writer) {
        this.writeImports(writer);
        this.statements.forEach((statement, idx) => {
            statement.write(writer);
            writer.newLine();
            if (idx < this.statements.length - 1) {
                writer.newLine();
            }
        });
    }
    /*******************************
     * Helper Methods
     *******************************/
    getImportName(reference) {
        const name = reference.name;
        const alias = reference.alias;
        return `${name}${alias ? ` as ${alias}` : ""}`;
    }
    writeImports(writer) {
        // Deduplicate references by their fully qualified paths
        const uniqueReferences = new Map();
        for (const reference of this.references) {
            const fullyQualifiedPath = reference.getFullyQualifiedModulePath();
            const existingRefs = uniqueReferences.get(fullyQualifiedPath);
            const referenceName = reference.name;
            if (existingRefs) {
                if (!existingRefs.referenceNames.has(referenceName)) {
                    existingRefs.references.push(reference);
                    existingRefs.referenceNames.add(referenceName);
                }
            }
            else {
                uniqueReferences.set(fullyQualifiedPath, {
                    modulePath: reference.modulePath,
                    references: [reference],
                    referenceNames: new Set([referenceName])
                });
            }
        }
        for (const [fullyQualifiedPath, { modulePath, references }] of uniqueReferences) {
            const refModulePath = modulePath;
            if (refModulePath[0] === this.path[0]) {
                // Relativize the import
                // Calculate the common prefix length
                let commonPrefixLength = 0;
                while (commonPrefixLength < this.path.length &&
                    commonPrefixLength < refModulePath.length &&
                    this.path[commonPrefixLength] === refModulePath[commonPrefixLength]) {
                    commonPrefixLength++;
                }
                // Calculate the number of levels to go up
                let levelsUp = this.path.length - commonPrefixLength;
                // If this is an __init__.py file, then we must go one more level up.
                if (this.isInitFile) {
                    levelsUp++;
                }
                // Build the relative import path
                let relativePath = levelsUp > 0 ? ".".repeat(levelsUp) : ".";
                relativePath += refModulePath.slice(commonPrefixLength).join(".");
                // Write the relative import statement
                writer.write(`from ${relativePath} import ${references.map((ref) => this.getImportName(ref)).join(", ")}`);
            }
            else {
                // Use fully qualified path
                writer.write(`from ${fullyQualifiedPath} import ${references.map((ref) => this.getImportName(ref)).join(", ")}`);
            }
            writer.newLine();
        }
        if (Object.keys(uniqueReferences).length > 0) {
            writer.newLine();
        }
    }
}
